@@ -11,11 +11,14 @@ class ComponentRegistry(object):
         self._registry = {}  # component name -> component_class mapping
 
     def register(self, name=None, component=None):
-        if name in self._registry:
+        existing_component = self._registry.get(name)
+        if (
+            existing_component
+            and existing_component.class_hash != component.class_hash
+        ):
             raise AlreadyRegistered(
-                'The component "%s" is already registered' % name
+                'The component "%s" has already been registered' % name
             )
-
         self._registry[name] = component
 
     def unregister(self, name):
@@ -34,3 +37,24 @@ class ComponentRegistry(object):
 
     def clear(self):
         self._registry = {}
+
+
+# This variable represents the global component registry
+registry = ComponentRegistry()
+
+
+def register(name):
+    """Class decorator to register a component.
+
+    Usage:
+
+    @register("my_component")
+    class MyComponent(component.Component):
+        ...
+    """
+
+    def decorator(component):
+        registry.register(name=name, component=component)
+        return component
+
+    return decorator
