@@ -1,6 +1,6 @@
 import re
-from textwrap import dedent
-from typing import Callable
+import textwrap
+from typing import Callable, Iterable, Optional
 
 from django.template import Context, Template, TemplateSyntaxError
 
@@ -77,6 +77,14 @@ class _DashboardComponent(component.Component):
     template_name = "slotted_component_nesting_template_pt2_dashboard.html"
 
 
+class ComponentWithDefaultSlot(component.Component):
+    template_name = "template_with_default_slot.html"
+
+
+class ComponentWithDefaultAndRequiredSlot(component.Component):
+    template_name = "template_with_default_and_required_slot.html"
+
+
 class ComponentTemplateTagTest(SimpleTestCase):
     def setUp(self):
         # NOTE: component.registry is global, so need to clear before each test
@@ -144,7 +152,7 @@ class ComponentTemplateTagTest(SimpleTestCase):
                 """Variable: <strong>variable</strong>\n"""
                 """Variable2: <strong>hej</strong>"""
             )
-            self.assertHTMLEqual(rendered, dedent(expected_outcome))
+            self.assertHTMLEqual(rendered, textwrap.dedent(expected_outcome))
 
     def test_component_called_with_singlequoted_name(self):
         component.registry.register(name="test", component=SimpleComponent)
@@ -349,6 +357,185 @@ class ComponentSlottedTemplateTagTest(SimpleTestCase):
         with self.assertRaises(TemplateSyntaxError):
             template.render(Context({}))
 
+    def test_default_slot_is_fillable_by_implicit_fill_content(self):
+        component.registry.register("test_comp", ComponentWithDefaultSlot)
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block 'test_comp' %}
+              <p>This fills the 'main' slot.</p>
+            {% endcomponent_block %}
+            """
+        )
+
+        expected = """
+        <div>
+          <main><p>This fills the 'main' slot.</p></main>
+        </div>
+        """
+        rendered = template.render(Context({}))
+        self.assertHTMLEqual(rendered, expected)
+
+    def test_default_slot_is_fillable_by_explicit_fill_content(self):
+        component.registry.register("test_comp", ComponentWithDefaultSlot)
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block 'test_comp' %}
+              {% fill "main" %}<p>This fills the 'main' slot.</p>{% endfill %}
+            {% endcomponent_block %}
+            """
+        )
+        expected = """
+        <div>
+          <main><p>This fills the 'main' slot.</p></main>
+        </div>
+        """
+        rendered = template.render(Context({}))
+        self.assertHTMLEqual(rendered, expected)
+
+    def test_error_raised_when_default_and_required_slot_not_filled(self):
+        component.registry.register(
+            "test_comp", ComponentWithDefaultAndRequiredSlot
+        )
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block 'test_comp' %}
+            {% endcomponent_block %}
+            """
+        )
+        with self.assertRaises(TemplateSyntaxError):
+            template.render(Context({}))
+
+    def test_fill_tag_can_occur_within_component_block_nested_in_implicit_fill(
+        self,
+    ):
+        component.registry.register("test_comp", ComponentWithDefaultSlot)
+        component.registry.register("slotted", SlottedComponent)
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block 'test_comp' %}
+              {% component_block "slotted" %}
+                {% fill "header" %}This Is Allowed{% endfill %}
+                {% fill "main" %}{% endfill %}
+                {% fill "footer" %}{% endfill %}
+              {% endcomponent_block %}
+            {% endcomponent_block %}
+            """
+        )
+        expected = """
+        <div>
+          <main>
+            <custom-template>
+              <header>This Is Allowed</header>
+              <main></main>
+              <footer></footer>
+            </custom-template>
+          </main>
+        </div>
+        """
+        rendered = template.render(Context({}))
+        self.assertHTMLEqual(rendered, expected)
+
+    def test_error_from_mixed_implicit_and_explicit_fill_content(self):
+        component.registry.register("test_comp", ComponentWithDefaultSlot)
+
+        with self.assertRaises(TemplateSyntaxError):
+            Template(
+                """
+                {% load component_tags %}
+                {% component_block 'test_comp' %}
+                  {% fill "main" %}Main content{% endfill %}
+                  <p>And add this too!</p>
+                {% endcomponent_block %}
+                """
+            )
+
+    def test_comments_permitted_inside_implicit_fill_content(self):
+        component.registry.register("test_comp", ComponentWithDefaultSlot)
+        Template(
+            """
+            {% load component_tags %}
+            {% component_block 'test_comp' %}
+              <p>Main Content</p>
+              {% comment %}
+              This won't show up in the rendered HTML
+              {% endcomment %}
+              {# Nor will this #}
+            {% endcomponent_block %}
+            """
+        )
+        self.assertTrue(True)
+
+    def test_component_without_default_slot_refuses_implicit_fill(self):
+        component.registry.register("test_comp", SlottedComponent)
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block 'test_comp' %}
+              <p>This shouldn't work because the included component doesn't mark
+              any of its slots as 'default'</p>
+            {% endcomponent_block %}
+            """
+        )
+        with self.assertRaises(TemplateSyntaxError):
+            template.render(Context({}))
+
+    def test_component_template_cannot_have_multiple_default_slots(self):
+        class BadComponent(component.Component):
+            def get_template(
+                self, context, template_name: Optional[str] = None
+            ) -> Template:
+                return Template(
+                    """
+                    {% load django_components %}
+                    <div>
+                    {% slot "icon" %} {% endslot default %}
+                    {% slot "description" %} {% endslot default %}
+                    </div>
+                    """
+                )
+
+        c = BadComponent("name")
+        with self.assertRaises(TemplateSyntaxError):
+            c.render(Context({}))
+
+    def test_slot_name_fill_typo_gives_helpful_error_message(self):
+        component.registry.register(name="test1", component=SlottedComponent)
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block "test1" %}
+                {% fill "haeder" %}
+                    Custom header
+                {% endfill %}
+                {% fill "main" %}
+                    main content
+                {% endfill %}
+            {% endcomponent_block %}
+        """
+        )
+        with self.assertRaises(TemplateSyntaxError):
+            try:
+                template.render(Context({}))
+            except TemplateSyntaxError as e:
+                self.assertEqual(
+                    textwrap.dedent(
+                        """\
+                        Component 'test1' passed fill that refers to undefined slot: 'haeder'.
+                        Unfilled slot names are: ['footer', 'header'].
+                        Did you mean 'header'?"""
+                    ),
+                    str(e),
+                )
+                raise e
+
 
 class SlottedTemplateRegressionTests(SimpleTestCase):
     def setUp(self):
@@ -552,7 +739,7 @@ class NestedSlotTests(SimpleTestCase):
         super().tearDownClass()
         component.registry.clear()
 
-    def test_default_slots_render_correctly(self):
+    def test_default_slot_contents_render_correctly(self):
         template = Template(
             """
             {% load component_tags %}
@@ -779,27 +966,31 @@ class TemplateSyntaxErrorTests(SimpleTestCase):
         super().tearDownClass()
         component.registry.clear()
 
-    def test_variable_outside_fill_tag_is_error(self):
-        with self.assertRaises(TemplateSyntaxError):
-            Template(
-                """
-                {% load component_tags %}
-                {% component_block "test" %}
-                    {{ anything }}
-                {% endcomponent_block %}
+    def test_variable_outside_fill_tag_compiles_w_out_error(self):
+        # As of v0.28 this is valid, provided the component registered under "test"
+        # contains a slot tag marked as 'default'. This is verified outside
+        # template compilation time.
+        Template(
             """
-            )
+            {% load component_tags %}
+            {% component_block "test" %}
+                {{ anything }}
+            {% endcomponent_block %}
+            """
+        )
 
-    def test_text_outside_fill_tag_is_error(self):
-        with self.assertRaises(TemplateSyntaxError):
-            Template(
-                """
-                {% load component_tags %}
-                {% component_block "test" %}
-                    Text
-                {% endcomponent_block %}
+    def test_text_outside_fill_tag_is_not_error(self):
+        # As of v0.28 this is valid, provided the component registered under "test"
+        # contains a slot tag marked as 'default'. This is verified outside
+        # template compilation time.
+        Template(
             """
-            )
+            {% load component_tags %}
+            {% component_block "test" %}
+                Text
+            {% endcomponent_block %}
+            """
+        )
 
     def test_nonfill_block_outside_fill_tag_is_error(self):
         with self.assertRaises(TemplateSyntaxError):
@@ -843,7 +1034,7 @@ class TemplateSyntaxErrorTests(SimpleTestCase):
                     {% fill "main" %}Custom main{% endfill %}
                     {% fill "footer" %}Custom footer{% endfill %}
                 {% endcomponent_block %}
-            """
+                """
             ).render(Context({}))
 
     def test_non_unique_fill_names_is_error(self):
@@ -855,7 +1046,7 @@ class TemplateSyntaxErrorTests(SimpleTestCase):
                     {% fill "header" %}Custom header {% endfill %}
                     {% fill "header" %}Other header{% endfill %}
                 {% endcomponent_block %}
-            """
+                """
             ).render(Context({}))
 
     def test_non_unique_slot_names_is_error(self):
@@ -1055,3 +1246,305 @@ class ConditionalIfFilledSlotsTests(SimpleTestCase):
         """
         rendered = Template(template).render(Context({}))
         self.assertHTMLEqual(rendered, expected)
+
+
+class RegressionTests(SimpleTestCase):
+    """Ensure we don't break the same thing AGAIN."""
+
+    def setUp(self):
+        component.registry.clear()
+        super().setUp()
+
+    @classmethod
+    def tearDownClass(cls):
+        super().tearDownClass()
+        component.registry.clear()
+
+    def test_block_and_extends_tag_works(self):
+        component.registry.register("slotted_component", SlottedComponent)
+        template = """
+        {% extends "extendable_template_with_blocks.html" %}
+        {% load component_tags %}
+        {% block body %}
+          {% component_block "slotted_component" %}
+            {% fill "header" %}{% endfill %}
+            {% fill "main" %}
+              TEST
+            {% endfill %}
+            {% fill "footer" %}{% endfill %}
+          {% endcomponent_block %}
+        {% endblock %}
+        """
+        rendered = Template(template).render(Context())
+        expected = """
+        <!DOCTYPE html>
+        <html lang="en">
+        <body>
+        <main role="main">
+          <div class='container main-container'>
+            <custom-template>
+              <header></header>
+              <main>TEST</main>
+              <footer></footer>
+            </custom-template>
+          </div>
+        </main>
+        </body>
+        </html>
+        """
+        self.assertHTMLEqual(rendered, expected)
+
+
+class IterationFillTest(SimpleTestCase):
+    """Tests a behaviour of {% fill .. %} tag which is inside a template {% for .. %} loop."""
+
+    class ComponentSimpleSlotInALoop(django_components.component.Component):
+        template_name = "template_with_slot_in_a_loop.html"
+
+        def get_context_data(self, objects: Iterable) -> dict:
+            return {
+                "objects": objects,
+            }
+
+    def setUp(self):
+        django_components.component.registry.clear()
+
+    def test_inner_slot_iteration_basic(self):
+        component.registry.register(
+            "slot_in_a_loop", self.ComponentSimpleSlotInALoop
+        )
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block "slot_in_a_loop" objects=objects %}
+                {% fill "slot_inner" %}
+                    {{ object }}
+                {% endfill %}
+            {% endcomponent_block %}
+        """
+        )
+        objects = ["OBJECT1", "OBJECT2"]
+        rendered = template.render(Context({"objects": objects}))
+
+        self.assertHTMLEqual(
+            rendered,
+            """
+            OBJECT1
+            OBJECT2
+            """,
+        )
+
+    def test_inner_slot_iteration_with_variable_from_outer_scope(self):
+        component.registry.register(
+            "slot_in_a_loop", self.ComponentSimpleSlotInALoop
+        )
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block "slot_in_a_loop" objects=objects %}
+                {% fill "slot_inner" %}
+                    {{ outer_scope_variable }}
+                    {{ object }}
+                {% endfill %}
+            {% endcomponent_block %}
+        """
+        )
+        objects = ["OBJECT1", "OBJECT2"]
+        rendered = template.render(
+            Context(
+                {
+                    "objects": objects,
+                    "outer_scope_variable": "OUTER_SCOPE_VARIABLE",
+                }
+            )
+        )
+
+        self.assertHTMLEqual(
+            rendered,
+            """
+            OUTER_SCOPE_VARIABLE
+            OBJECT1
+            OUTER_SCOPE_VARIABLE
+            OBJECT2
+            """,
+        )
+
+    def test_inner_slot_iteration_nested(self):
+        component.registry.register(
+            "slot_in_a_loop", self.ComponentSimpleSlotInALoop
+        )
+
+        objects = [
+            {"inner": ["OBJECT1_ITER1", "OBJECT2_ITER1"]},
+            {"inner": ["OBJECT1_ITER2", "OBJECT2_ITER2"]},
+        ]
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block "slot_in_a_loop" objects=objects %}
+                {% fill "slot_inner" %}
+                    {% component_block "slot_in_a_loop" objects=object.inner %}
+                        {% fill "slot_inner" %}
+                            {{ object }}
+                        {% endfill %}
+                    {% endcomponent_block %}
+                {% endfill %}
+            {% endcomponent_block %}
+        """
+        )
+        rendered = template.render(Context({"objects": objects}))
+
+        self.assertHTMLEqual(
+            rendered,
+            """
+            OBJECT1_ITER1
+            OBJECT2_ITER1
+            OBJECT1_ITER2
+            OBJECT2_ITER2
+            """,
+        )
+
+    def test_inner_slot_iteration_nested_with_outer_scope_variable(self):
+        component.registry.register(
+            "slot_in_a_loop", self.ComponentSimpleSlotInALoop
+        )
+
+        objects = [
+            {"inner": ["OBJECT1_ITER1", "OBJECT2_ITER1"]},
+            {"inner": ["OBJECT1_ITER2", "OBJECT2_ITER2"]},
+        ]
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block "slot_in_a_loop" objects=objects %}
+                {% fill "slot_inner" %}
+                    {{ outer_scope_variable_1 }}
+                    {% component_block "slot_in_a_loop" objects=object.inner %}
+                        {% fill "slot_inner" %}
+                            {{ outer_scope_variable_2 }}
+                            {{ object }}
+                        {% endfill %}
+                    {% endcomponent_block %}
+                {% endfill %}
+            {% endcomponent_block %}
+        """
+        )
+        rendered = template.render(
+            Context(
+                {
+                    "objects": objects,
+                    "outer_scope_variable_1": "OUTER_SCOPE_VARIABLE1",
+                    "outer_scope_variable_2": "OUTER_SCOPE_VARIABLE2",
+                }
+            )
+        )
+
+        self.assertHTMLEqual(
+            rendered,
+            """
+            OUTER_SCOPE_VARIABLE1
+            OUTER_SCOPE_VARIABLE2
+            OBJECT1_ITER1
+            OUTER_SCOPE_VARIABLE2
+            OBJECT2_ITER1
+            OUTER_SCOPE_VARIABLE1
+            OUTER_SCOPE_VARIABLE2
+            OBJECT1_ITER2
+            OUTER_SCOPE_VARIABLE2
+            OBJECT2_ITER2
+            """,
+        )
+
+    def test_inner_slot_iteration_nested_with_slot_default(self):
+        component.registry.register(
+            "slot_in_a_loop", self.ComponentSimpleSlotInALoop
+        )
+
+        objects = [
+            {"inner": ["OBJECT1_ITER1", "OBJECT2_ITER1"]},
+            {"inner": ["OBJECT1_ITER2", "OBJECT2_ITER2"]},
+        ]
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block "slot_in_a_loop" objects=objects %}
+                {% fill "slot_inner" %}
+                    {% component_block "slot_in_a_loop" objects=object.inner %}
+                        {% fill "slot_inner" as "super_slot_inner" %}
+                            {{ super_slot_inner.default }}
+                        {% endfill %}
+                    {% endcomponent_block %}
+                {% endfill %}
+            {% endcomponent_block %}
+        """
+        )
+        rendered = template.render(Context({"objects": objects}))
+
+        self.assertHTMLEqual(
+            rendered,
+            """
+            OBJECT1_ITER1 default
+            OBJECT2_ITER1 default
+            OBJECT1_ITER2 default
+            OBJECT2_ITER2 default
+            """,
+        )
+
+    def test_inner_slot_iteration_nested_with_slot_default_and_outer_scope_variable(
+        self,
+    ):
+        component.registry.register(
+            "slot_in_a_loop", self.ComponentSimpleSlotInALoop
+        )
+
+        objects = [
+            {"inner": ["OBJECT1_ITER1", "OBJECT2_ITER1"]},
+            {"inner": ["OBJECT1_ITER2", "OBJECT2_ITER2"]},
+        ]
+
+        template = Template(
+            """
+            {% load component_tags %}
+            {% component_block "slot_in_a_loop" objects=objects %}
+                {% fill "slot_inner" %}
+                    {{ outer_scope_variable_1 }}
+                    {% component_block "slot_in_a_loop" objects=object.inner %}
+                        {% fill "slot_inner" as "super_slot_inner" %}
+                            {{ outer_scope_variable_2 }}
+                            {{ super_slot_inner.default }}
+                        {% endfill %}
+                    {% endcomponent_block %}
+                {% endfill %}
+            {% endcomponent_block %}
+        """
+        )
+        rendered = template.render(
+            Context(
+                {
+                    "objects": objects,
+                    "outer_scope_variable_1": "OUTER_SCOPE_VARIABLE1",
+                    "outer_scope_variable_2": "OUTER_SCOPE_VARIABLE2",
+                }
+            )
+        )
+
+        self.assertHTMLEqual(
+            rendered,
+            """
+            OUTER_SCOPE_VARIABLE1
+            OUTER_SCOPE_VARIABLE2
+            OBJECT1_ITER1 default
+            OUTER_SCOPE_VARIABLE2
+            OBJECT2_ITER1 default
+            OUTER_SCOPE_VARIABLE1
+            OUTER_SCOPE_VARIABLE2
+            OBJECT1_ITER2 default
+            OUTER_SCOPE_VARIABLE2
+            OBJECT2_ITER2 default
+            """,
+        )
